﻿StructuredBuffer<float2> pivotBuffer : register(s0);
StructuredBuffer<float2> rotationBuffer : register(s1);
RWStructuredBuffer<float2> vertexOutputBuffer : register(u0);
RWStructuredBuffer<float2> texCoordBuffer : register(u1);  // Find better solution. Integer is sufficent here.

static const float PI = 3.14159265f;

cbuffer inputs : register(c0){
	int run;
	int mode;
	int groupsX;
	int groupsY;
};

cbuffer powers : register(c1){
		float4 gPowers[8];
};


//int FindNextPowerOfLog(int number, int mode)
//{
//	float logMode = log2(number) / log2(mode);
//
//	return round(pow(mode, floor(round(logMode))));
//}


//float FindNextPowerOf(float number, float mode)
//{
//	float currentPower = mode;
//	float exp = 2;
//	float nextPower = pow(mode, exp);
//
//	while (number >= nextPower)
//	{
//		currentPower = nextPower;
//		nextPower = pow(mode, ++exp);
//	}
//
//	return currentPower;
//
//}

//float FindPAmount(float number, float mode)
//{
//	float currentPower = mode;
//	float exp = 2;
//	float nextPower = pow(mode, exp);
//	float powers = 1;
//
//	while (number >= nextPower)
//	{
//		currentPower = nextPower;
//		nextPower = pow(mode, ++exp);
//		powers++;
//	}
//
//	return powers;
//
//}

float2 RotateAroundPoint(float2 pos, float2 pivot, float angle)
{
	float2 centered = pos - pivot;

		float2 rotated;

	rotated.x = (centered.x * cos(angle) - centered.y * sin(angle) + pivot.x);
	rotated.y = (centered.x * sin(angle) + centered.y * cos(angle) + pivot.y);

	return rotated;

}

float2 ComputeTileCoordinates(int iteration, int mode)
{
	int pivotOffset = 1;
	int modulus = mode;
	int oldMod = mode;

	float2 tileCoord = float2(1, 1);

	float increment = 2 * PI / mode;

	float logMode = log2(iteration) / log2(mode);

    int iterations = (int)floor(round(logMode));

	tileCoord = RotateAroundPoint(tileCoord, pivotBuffer[pivotOffset], -(iteration%modulus) * increment);

	while (iterations > 0)
	{
		pivotOffset++;
		modulus *= mode;
		float rotationAngle = ((iteration % modulus)/oldMod) * increment;
		tileCoord = RotateAroundPoint(tileCoord, pivotBuffer[pivotOffset], -rotationAngle);
		oldMod = modulus;
		iterations--;
	}

	return tileCoord;
}

float FindNextPowerOf(float number)
{
	int index = 0;

	float start = gPowers[index / 8][index % 8];

	while (number >= start)
	{
		index++;

		start = gPowers[index / 8][index % 8];
	}

	return start;
}


int ComputeRotationRep(int iteration, int mode)
{
	int finalRotation = 0;

	int divisor = mode;

	if (iteration > mode)
		divisor = FindNextPowerOf(iteration);
		

	while (iteration >= mode)
	{
		finalRotation += iteration / divisor;
		iteration %= divisor;
		divisor /= mode;

	}

	finalRotation += iteration;

	finalRotation %= mode;

	return finalRotation;



}


[numthreads(32, 32, 1)]
void main( uint3 groupId : SV_GroupID, uint groupIndex : SV_GroupIndex)
{
	int currentRun = run;
	int threads = 1024;

	int indexMax = (threads * groupsX * groupsY);

	int tileIndex = (int)groupIndex + (int)groupId.x * threads + groupsX * groupId.y * threads + currentRun * indexMax;

	int vBufferIndex = (indexMax  - 1 - ((int)groupIndex + (int)groupId.x * threads + (groupsX * groupId.y * threads))) * 6;
	

	for (int j = 0; j < 6; j++)
		vertexOutputBuffer[vBufferIndex + j] = ComputeTileCoordinates(tileIndex, mode) + rotationBuffer[ComputeRotationRep(tileIndex, mode) * 6 + j];

	texCoordBuffer[vBufferIndex + 0] = float2(-0.5f, -0.5f);
	texCoordBuffer[vBufferIndex + 1] = float2(0.5f, 0.5f);
	texCoordBuffer[vBufferIndex + 2] = float2(-0.5f, 0.5f);
	texCoordBuffer[vBufferIndex + 3] = float2(0.5f, -0.5f);
	texCoordBuffer[vBufferIndex + 4] = float2(0.5f, 0.5f);
	texCoordBuffer[vBufferIndex + 5] = float2(-0.5f, -0.5f);

}

