Texture2D triangleTexture; //Triangle Texture
Texture2D lodTexture; //Texture for LOD Rendering

Texture2D maskTexture;

bool enableAlpha = true;

bool useMaskTexture = false;

float4x4 mvp;

float4x4 lodMvp; //Matrix for zooming and translating textures...

float aspectX = 1;
float aspectY = 1;


SamplerState TriangleTextureSampler
{
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = Clamp;
	AddressV = Clamp;

};

SamplerState AlphaSampler
{
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = Clamp;
	AddressV = Clamp;
	MaxLod = 1;

};



#ifndef DX10

//Pre-multiplied Alpha Blend State

BlendState blendStates[2]
{
	{
		BlendEnable[0] = TRUE;
		SrcBlend[0] = ONE;
		DestBlend[0] = INV_SRC_ALPHA;
		BlendOp[0] = ADD; 
		SrcBlendAlpha[0] = ONE;
		DestBlendAlpha[0] = ONE;
		BlendOpAlpha[0] = ADD;

		RenderTargetWriteMask[0] = 0x0F;
	},
	{
		BlendEnable[0] = FALSE;
	}
};


#else

BlendState DX10BlendState
{
	BlendEnable[0] = TRUE;
	SrcBlend = ONE;
	DestBlend = INV_SRC_ALPHA;
	BlendOp = ADD;
	SrcBlendAlpha = ONE;
	DestBlendAlpha = ONE;
	BlendOpAlpha = ADD;

	RenderTargetWriteMask[0] = 0x0F;
};

#endif


struct VS_IN
{
	float2 pos : POSITION;
	float2 TexCoord : TEXCOORD0;
};

struct PS_IN
{
	float4 pos : SV_POSITION;
    float2 TexCoord : TEXCOORD0;
	float2 TexCoordMask : TEXCOORD1;
};


PS_IN VS( VS_IN input )
{
	PS_IN output = (PS_IN)0;
	
	output.pos = mul(float4(input.pos.x,input.pos.y,1,1),mvp);

	output.TexCoord.x = input.TexCoord.x / aspectX + 0.5;

	output.TexCoord.y = input.TexCoord.y / aspectY + 0.5;

	output.TexCoordMask.x = input.TexCoord.x + 0.5;
	output.TexCoordMask.y = input.TexCoord.y + 0.5;

	return output;
}


float4 ComputeColor(float4 color)
{

	if (!enableAlpha)
	{
		color.rgb = color.rgb + float3(1, 1, 1)*(1 - color.a);
		color.a = 1;
	}

	return color;
}

float4 PS( PS_IN input ) : SV_Target
{
	bool texCoordOutOfRange = input.TexCoord.x < 0 || input.TexCoord.y < 0 || input.TexCoord.y > 1 || input.TexCoord.x > 1;
		
	if (useMaskTexture)
	{
		float4 color = triangleTexture.Sample(TriangleTextureSampler, input.TexCoord).rgba;
			
		float alphaTex = (1 - maskTexture.Sample(AlphaSampler, input.TexCoordMask).a);

		if (texCoordOutOfRange)
			color = float4(1, 1, 1, 1);

		return ComputeColor(color) * alphaTex;
	}
	else if (!useMaskTexture && !texCoordOutOfRange)
	{
		float4 color = triangleTexture.Sample(TriangleTextureSampler, input.TexCoord);
		
		return ComputeColor(color);
	}
	else
		return float4(0, 0, 0, 0);

}


PS_IN VSLODTexture(VS_IN input)
{
	PS_IN output = (PS_IN)0;

	output.pos = mul(float4(input.pos.x, input.pos.y, 1, 1), lodMvp);

	output.TexCoord = input.TexCoord;
	
	return output;
}

float4 PSLODTexture(PS_IN input) : SV_Target
{
	
	return lodTexture.Sample(TriangleTextureSampler, input.TexCoord);
}

#ifndef DX10

technique11 Render
{
	pass P0
	{
		
		SetGeometryShader(0);
		SetVertexShader(CompileShader(vs_5_0, VS()));
		SetPixelShader(CompileShader(ps_5_0, PS()));
		SetBlendState(blendStates[0], float4(0.0f, 0.0f, 0.0f,0.0f), 0xffffffff);
	}
}

technique11 RenderLODTexture
{
	pass P0
	{
		SetGeometryShader(0);
		SetVertexShader(CompileShader(vs_5_0, VSLODTexture()));
		SetPixelShader(CompileShader(ps_5_0, PSLODTexture()));
		SetBlendState(blendStates[1], float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
	}
}

#else

technique10 RenderDX10
{
	pass P0
	{

		SetGeometryShader(0);
		SetVertexShader(CompileShader(vs_4_0, VS()));
		SetPixelShader(CompileShader(ps_4_0, PS()));
		SetBlendState(DX10BlendState, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
	}
}


#endif